"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = __importDefault(require("assert"));
const autonomies_1 = require("./autonomies");
const provinces_1 = require("./provinces");
const cities_1 = require("./cities");
const test = ({ name, fn }) => {
    try {
        fn();
        console.log(`✅ ${name}`);
    }
    catch (error) {
        console.error(`❌ ${name}`);
        console.error(error);
    }
};
const groupByCode = (items) => {
    const seenCodes = {};
    items.forEach((item) => {
        if (!(item.code in seenCodes)) {
            seenCodes[item.code] = 0;
        }
        seenCodes[item.code] += 1;
    });
    Object.entries(seenCodes).forEach(([id, count]) => {
        if (count > 1) {
            (0, assert_1.default)(false, `code "${id}" is repeted ${count} times`);
        }
    });
};
console.group("Testing autonomies.ts:");
test({
    name: "should return all 19 autonomies",
    fn: () => {
        const result = (0, autonomies_1.autonomies)();
        assert_1.default.strictEqual(result.length, 19);
    },
});
test({
    name: "should return all autonomies with their provinces and cities",
    fn: () => {
        const result = (0, autonomies_1.autonomies)({ with_provinces: true, with_cities: true });
        (0, assert_1.default)(result.every((item) => Array.isArray(item.provinces) && Array.isArray(item.cities)));
    },
});
test({
    name: "should return all autonomies without their provinces and cities",
    fn: () => {
        const result = (0, autonomies_1.autonomies)({ with_provinces: false, with_cities: false });
        (0, assert_1.default)(result.every((item) => (item === null || item === void 0 ? void 0 : item.provinces) === undefined && (item === null || item === void 0 ? void 0 : item.cities) === undefined));
    },
});
test({
    name: 'should return 1 autonomy ("Extremadura") when code is "11"',
    fn: () => {
        const result = (0, autonomies_1.autonomies)({ code: 11 });
        assert_1.default.strictEqual(result.length, 1);
        assert_1.default.deepStrictEqual(result[0], {
            code: "11",
            name: "Extremadura",
            flag: "https://upload.wikimedia.org/wikipedia/commons/1/13/Flag_of_Extremadura%2C_Spain_%28with_coat_of_arms%29.svg",
            coat_of_arms: "https://upload.wikimedia.org/wikipedia/commons/e/e6/Escudo_de_Extremadura.svg",
        });
    },
});
test({
    name: 'should return 2 autonomies ("Castilla y León" and "Castilla-La Mancha") when name is "tilla"',
    fn: () => {
        const result = (0, autonomies_1.autonomies)({ name: "tilla" });
        assert_1.default.deepStrictEqual(result[0], {
            code: "07",
            name: "Castilla y León",
            flag: "https://upload.wikimedia.org/wikipedia/commons/1/13/Flag_of_Castile_and_Le%C3%B3n.svg",
            coat_of_arms: "https://upload.wikimedia.org/wikipedia/commons/2/24/Escudo_de_Castilla_y_Le%C3%B3n_-_Versi%C3%B3n_her%C3%A1ldica_oficial.svg",
        });
        assert_1.default.deepStrictEqual(result[1], {
            code: "08",
            name: "Castilla-La Mancha",
            flag: "https://upload.wikimedia.org/wikipedia/commons/d/d4/Bandera_Castilla-La_Mancha.svg",
            coat_of_arms: "https://upload.wikimedia.org/wikipedia/commons/e/eb/Escudo_de_Castilla-La_Mancha.svg",
        });
        assert_1.default.strictEqual(result.length, 2);
    },
});
test({
    name: 'should return empty array when code is "DOES_NOT_EXIST"',
    fn: () => {
        const result = (0, autonomies_1.autonomies)({ code: "DOES_NOT_EXIST" });
        assert_1.default.strictEqual(result.length, 0);
    },
});
console.groupEnd();
console.group("\nTesting provinces.ts:");
test({
    name: "should return all 52 provinces",
    fn: () => {
        const result = (0, provinces_1.provinces)();
        assert_1.default.strictEqual(result.length, 52);
    },
});
test({
    name: "should return all provinces with their autonomy and cities",
    fn: () => {
        const result = (0, provinces_1.provinces)({ with_autonomy: true, with_cities: true });
        (0, assert_1.default)(result.every((item) => (item === null || item === void 0 ? void 0 : item.autonomy) !== undefined && (item === null || item === void 0 ? void 0 : item.cities) !== undefined));
    },
});
test({
    name: "should return all provinces without their autonomy and cities",
    fn: () => {
        const result = (0, provinces_1.provinces)({ with_autonomy: false, with_cities: false });
        (0, assert_1.default)(result.every((item) => (item === null || item === void 0 ? void 0 : item.autonomy) === undefined && (item === null || item === void 0 ? void 0 : item.cities) === undefined));
    },
});
test({
    name: 'should return 1 provinces ("Burgos") when code is 09',
    fn: () => {
        const result = (0, provinces_1.provinces)({ code: "09" });
        assert_1.default.strictEqual(result.length, 1);
        assert_1.default.deepStrictEqual(result[0], {
            code: "09",
            name: "Burgos",
            code_autonomy: "07",
            flag: "https://upload.wikimedia.org/wikipedia/commons/b/b5/Flag_Burgos_Province.svg",
            coat_of_arms: "https://upload.wikimedia.org/wikipedia/commons/d/d2/Escudo_de_la_Provincia_de_Burgos.svg",
        });
    },
});
test({
    name: 'should return 2 provinces when code_autonomy is "11"',
    fn: () => {
        const result = (0, provinces_1.provinces)({ code_autonomy: "11" });
        assert_1.default.strictEqual(result.length, 2);
        assert_1.default.deepStrictEqual(result[0], {
            code: "06",
            name: "Badajoz",
            code_autonomy: "11",
            flag: "https://upload.wikimedia.org/wikipedia/commons/8/8c/Provincia_de_Badajoz_-_Bandera.svg",
            coat_of_arms: "https://upload.wikimedia.org/wikipedia/commons/a/a7/Provincia_de_Badajoz_-_Escudo.svg",
        });
        assert_1.default.deepStrictEqual(result[1], {
            code: "10",
            name: "Cáceres",
            code_autonomy: "11",
            flag: "https://upload.wikimedia.org/wikipedia/commons/9/95/Flag_of_the_province_of_C%C3%A1ceres.svg",
            coat_of_arms: "https://upload.wikimedia.org/wikipedia/commons/4/48/Escudo_de_la_Diputaci%C3%B3n_de_C%C3%A1ceres.svg",
        });
    },
});
test({
    name: 'should return 6 provinces when name is "Ri"',
    fn: () => {
        const result = (0, provinces_1.provinces)({ name: "Ri" });
        assert_1.default.strictEqual(result.length, 6);
    },
});
test({
    name: 'should return empty array when code is "DOES_NOT_EXIST"',
    fn: () => {
        const result = (0, provinces_1.provinces)({ code: "DOES_NOT_EXIST" });
        assert_1.default.strictEqual(result.length, 0);
    },
});
test({
    name: 'should return empty array when code_autonomy is "DOES_NOT_EXIST"',
    fn: () => {
        const result = (0, provinces_1.provinces)({ code_autonomy: "DOES_NOT_EXIST" });
        assert_1.default.strictEqual(result.length, 0);
    },
});
console.groupEnd();
console.group("\nTesting cities.ts:");
test({
    name: "should return all 8131 cities",
    fn: () => {
        const result = (0, cities_1.cities)();
        assert_1.default.strictEqual(result.length, 8131);
    },
});
test({
    name: "should return all cities with their autonomy and province",
    fn: () => {
        const result = (0, cities_1.cities)({ with_autonomy: true, with_province: true });
        (0, assert_1.default)(result.every((item) => (item === null || item === void 0 ? void 0 : item.autonomy) !== undefined && (item === null || item === void 0 ? void 0 : item.province) !== undefined));
    },
});
test({
    name: "should return all cities without their autonomy and province",
    fn: () => {
        const result = (0, cities_1.cities)({ with_autonomy: false, with_province: false });
        (0, assert_1.default)(result.every((item) => (item === null || item === void 0 ? void 0 : item.autonomy) === undefined && (item === null || item === void 0 ? void 0 : item.province) === undefined));
    },
});
test({
    name: 'should return 1 city ("San Nicolás del Puerto") when code is "410883"',
    fn: () => {
        const result = (0, cities_1.cities)({ code: "410883" });
        assert_1.default.strictEqual(result.length, 1);
        assert_1.default.deepStrictEqual(result[0], {
            code: "410883",
            name: "San Nicolás del Puerto",
            code_autonomy: "01",
            code_province: "41",
            flag: null,
            coat_of_arms: null,
        });
    },
});
test({
    name: 'should return 179 cities of Madrid when code_autonomy is "13"',
    fn: () => {
        const result = (0, cities_1.cities)({ code_autonomy: "13" });
        (0, assert_1.default)(result.every((item) => item.code_autonomy === "13"));
        assert_1.default.strictEqual(result.length, 179);
    },
});
test({
    name: 'should return 141 cities when code_province is "03"',
    fn: () => {
        const result = (0, cities_1.cities)({ code_province: "03" });
        (0, assert_1.default)(result.every((item) => item.code_province === "03"));
        assert_1.default.strictEqual(result.length, 141);
    },
});
test({
    name: 'should return 25 cities when name is "Valverde"',
    fn: () => {
        const result = (0, cities_1.cities)({ name: "Valverde" });
        assert_1.default.strictEqual(result.length, 25);
    },
});
test({
    name: 'should return empty array when code is "DOES_NOT_EXIST"',
    fn: () => {
        const result = (0, cities_1.cities)({ code: "DOES_NOT_EXIST" });
        assert_1.default.strictEqual(result.length, 0);
    },
});
test({
    name: 'should return empty array when code_autonomy is "DOES_NOT_EXIST"',
    fn: () => {
        const result = (0, cities_1.cities)({ code_autonomy: "DOES_NOT_EXIST" });
        assert_1.default.strictEqual(result.length, 0);
    },
});
test({
    name: 'should return empty array when code_province is "DOES_NOT_EXIST"',
    fn: () => {
        const result = (0, cities_1.cities)({ code_province: "DOES_NOT_EXIST" });
        assert_1.default.strictEqual(result.length, 0);
    },
});
console.groupEnd();
console.group("\nTesting general things:");
test({
    name: "autonomies must have unique code",
    fn: () => {
        groupByCode((0, autonomies_1.autonomies)());
    },
});
test({
    name: "provinces must have unique code",
    fn: () => {
        groupByCode((0, autonomies_1.autonomies)());
    },
});
test({
    name: "cities must have unique code",
    fn: () => {
        groupByCode((0, cities_1.cities)());
    },
});
